import itertools
import math
import os
import shutil
import unittest

import numpy as np
import pandas as pd
from common_maths import *


#################################################################################
class TestCreateDir(unittest.TestCase):
    """TestCreateDir"""

    def setUp(self):
        """在每个测试之前运行的代码，用于设置测试环境"""
        self.test_dir = "test_create_dir"
        self.sub_dir = "subfolder"
        self.test_path = os.path.join(self.test_dir, self.sub_dir)

        # 确保测试目录是干净的
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)
        os.makedirs(self.test_dir)

    def tearDown(self):
        """在每个测试之后运行的代码，用于清理测试环境"""
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_create_new_directory(self):
        """测试函数在不存在的路径中创建目录"""
        new_dir_name = "new_folder"
        create_dir(new_dir_name, self.test_dir)
        full_path = os.path.join(self.test_dir, new_dir_name)
        self.assertTrue(os.path.exists(full_path), "目录未创建成功")

    def test_directory_already_exists(self):
        """测试函数在目录已经存在的情况下处理"""
        existing_dir_name = "existing_folder"
        os.makedirs(os.path.join(self.test_dir, existing_dir_name))
        create_dir(existing_dir_name, self.test_dir)
        # 验证目录是否仍然存在，且没有抛出异常
        full_path = os.path.join(self.test_dir, existing_dir_name)
        self.assertTrue(os.path.exists(full_path), "目录应该已经存在")


#######################################################################


class TestGetMaxDiff(unittest.TestCase):
    def test_standard_case(self):
        """测试函数在正常二维数组中的行为"""
        array = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
        expected = np.array([6, 6, 6])  # 每列的最大值与最小值的差
        result = get_max_diff(array)
        np.testing.assert_array_equal(result, expected)

    # def test_empty_array(self):
    #     """测试函数在空二维数组中的行为"""
    #     array = []
    #     expected = np.array([])
    #     result = get_max_diff(array)
    #     np.testing.assert_array_equal(result, expected)

    def test_single_column_array(self):
        """测试函数在只有一列的二维数组中的行为"""
        array = [[1], [3], [5]]
        expected = np.array([4])  # 只有一列，最大值与最小值的差
        result = get_max_diff(array)
        np.testing.assert_array_equal(result, expected)

    def test_invalid_input(self):
        """测试函数在非二维数组输入中的行为"""
        with self.assertRaises(ValueError):
            get_max_diff([1, 2, 3])  # 一维数组

        with self.assertRaises(ValueError):
            get_max_diff([[1, 2], [3, 4], [5]])  # 不规则的二维数组

    def test_non_numeric_data(self):
        """测试函数在包含非数字数据的二维数组中的行为"""
        array = [[1, 2, "a"], [3, 4, "b"]]
        with self.assertRaises(TypeError):
            get_max_diff(array)


###########################################################################################
class TestRemoveOutliersIQR(unittest.TestCase):
    def test_remove_outliers_basic(self):
        """
        测试简单的二维数组，确保函数能正确去除异常值。
        """
        data = np.array(
            [
                [1, 2, 3, 4],
                [5, 5, 5, 5],
                [100, 200, 300, 1000],  # 明显的异常值
                [2, 3, 4, 5],
                [6, 7, 8, 9],
                [0, 0, 0, 0],
            ]
        )

        cleaned_data, valid_indices = remove_outliers_iqr(data)
        expected_data = np.array(
            [[1, 2, 3, 4], [5, 5, 5, 5], [2, 3, 4, 5], [6, 7, 8, 9], [0, 0, 0, 0]]
        )

        expected_indices = [0, 1, 3, 4, 5]

        np.testing.assert_array_equal(cleaned_data, expected_data)
        np.testing.assert_array_equal(valid_indices, expected_indices)

    def test_all_inliers(self):
        """
        测试当没有异常值时，所有数据都应该保留。
        """
        data = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [2, 3, 4, 5], [6, 7, 8, 9]])

        cleaned_data, valid_indices = remove_outliers_iqr(data)

        np.testing.assert_array_equal(cleaned_data, data)
        np.testing.assert_array_equal(valid_indices, [0, 1, 2, 3])

    def test_single_column(self):
        """
        测试只有一列数据的情况。
        """
        data = np.array([[1], [2], [100], [3], [4]])  # 异常值

        cleaned_data, valid_indices = remove_outliers_iqr(data)
        expected_data = np.array([[1], [2], [3], [4]])

        expected_indices = [0, 1, 3, 4]

        np.testing.assert_array_equal(cleaned_data, expected_data)
        np.testing.assert_array_equal(valid_indices, expected_indices)


##################################################################
class TestExportToCsv(unittest.TestCase):
    def setUp(self):
        """在每个测试之前运行的代码，用于设置测试环境"""
        self.test_dir = "test_output"
        self.test_file = "test_data"
        self.test_data = [[1, 2, 3], [4, 5, 6]]

        # 确保测试目录是干净的
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)
        os.makedirs(self.test_dir)

    def tearDown(self):
        """在每个测试之后运行的代码，用于清理测试环境"""
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_file_creation(self):
        """测试函数是否能够创建CSV文件"""
        export_to_csv(self.test_data, self.test_file, self.test_dir)
        file_path = os.path.join(self.test_dir, self.test_file + ".csv")
        self.assertTrue(os.path.exists(file_path), "CSV文件未创建成功")

    def test_data_written_correctly(self):
        """测试函数是否正确写入数据到CSV文件"""
        export_to_csv(self.test_data, self.test_file, self.test_dir)

        file_path = os.path.join(self.test_dir, self.test_file + ".csv")
        # 检查CSV内容是否正确
        df = pd.read_csv(file_path, header=None, index_col=False)
        self.assertTrue(df.equals(pd.DataFrame(self.test_data)), "CSV文件内容不匹配")

    def test_append_data(self):
        """测试函数在文件存在时是否能够正确追加数据"""
        export_to_csv(self.test_data, self.test_file, self.test_dir)
        additional_data = [[7, 8, 9]]
        export_to_csv(additional_data, self.test_file, self.test_dir)

        file_path = os.path.join(self.test_dir, self.test_file + ".csv")

        # 检查CSV内容是否包含追加的数据
        df = pd.read_csv(file_path, header=None, index_col=False)
        expected_data = self.test_data + additional_data
        self.assertTrue(df.equals(pd.DataFrame(expected_data)), "数据追加失败")

    # def test_empty_data(self):
    #     """测试函数在传入空数据时的处理"""
    #     try:
    #         export_to_csv([], self.test_file, self.test_dir)
    #     except Exception as e:
    #         self.fail(f"函数在处理空数据时抛出异常: {e}")

    #     file_path = os.path.join(self.test_dir, self.test_file + ".csv")

    #     self.assertTrue(os.path.exists(file_path), "CSV文件未创建")

    #     # 检查CSV文件内容
    #     df = pd.read_csv(file_path)
    #     self.assertTrue(df.empty, "CSV文件不应包含数据")


# TAG 20240818 更新


class TestGenerate2DCombinationsIter(unittest.TestCase):
    def test_generate_all_combinations(self):
        """
        测试正常情况下，生成所有二维数组组合的迭代器。
        """
        data_3d = [[[1, 2], [3, 4]], [[5, 6], [7, 8]]]
        expected_output = [
            [(1, 3), (1, 4), (2, 3), (2, 4)],
            [(5, 7), (5, 8), (6, 7), (6, 8)],
        ]
        result_iter = generate_2d_combinations_iter(data_3d)

        for expected, result in zip(expected_output, result_iter):
            self.assertEqual(list(result), expected)

    def test_generate_first_group_combinations(self):
        """
        测试只生成第一个二维数组组合的迭代器。
        """
        data_3d = [[[1, 2], [3, 4]], [[5, 6], [7, 8]]]
        expected_output = [(1, 3), (1, 4), (2, 3), (2, 4)]

        result_iter = generate_2d_combinations_iter(data_3d, test_first_group=True)
        self.assertEqual(list(next(result_iter)), expected_output)

    def test_empty_input(self):
        """
        测试传入空的三维数组。
        """
        data_3d = []
        with self.assertRaises(ValueError):
            # 生成器内部有异常的话只有在真正调用的时候才会捕获
            list(generate_2d_combinations_iter(data_3d))

    def test_invalid_input(self):
        """
        测试传入非三维数组，应该抛出 ValueError。
        """
        invalid_data = [[[1, 2], 3], [4, 5]]  # 第二层不是一个列表
        with self.assertRaises(ValueError):
            list(generate_2d_combinations_iter(invalid_data))

    def test_single_element_input(self):
        """
        测试仅包含一个元素的三维数组。
        """
        data_3d = [[[1], [2]]]
        expected_output = [(1, 2)]

        result_iter = generate_2d_combinations_iter(data_3d)
        self.assertEqual(list(next(result_iter)), expected_output)

    def test_single_layer_array(self):
        """
        测试包含一个二维数组的三维数组。
        """
        data_3d = [[[1, 2], [3, 4]]]
        expected_output = [(1, 3), (1, 4), (2, 3), (2, 4)]

        result_iter = generate_2d_combinations_iter(data_3d)
        self.assertEqual(list(next(result_iter)), expected_output)


class TestGenerateRowPermutations(unittest.TestCase):
    def test_valid_input(self):
        """
        测试正常输入的排列组合生成。
        """
        data = [[1, 2], [3, 4]]
        expected_output = [
            [[1, 2], [3, 4]],
            [[1, 2], [4, 3]],
            [[2, 1], [3, 4]],
            [[2, 1], [4, 3]],
        ]
        result = list(generate_row_permutations(data))
        self.assertEqual(result, expected_output)

    def test_single_row(self):
        """
        测试只有一行的二维数组。
        """
        data = [[1, 2, 3]]
        expected_output = [[list(p)] for p in itertools.permutations(data[0])]
        result = list(generate_row_permutations(data))
        self.assertEqual(result, expected_output)

    def test_empty_rows(self):
        """
        测试包含空行的二维数组。
        """
        with self.assertRaises(ValueError):
            list(generate_row_permutations([[], [1, 2]]))

    def test_empty_input(self):
        """
        测试空的二维数组。
        """
        with self.assertRaises(ValueError):
            list(generate_row_permutations([]))

    def test_empty_element_in_rows(self):
        """
        测试包含空元素的二维数组。
        """
        with self.assertRaises(ValueError):
            list(generate_row_permutations([[1, 2], []]))


class TestCalculateTotalPermutations(unittest.TestCase):
    def test_valid_input(self):
        """
        测试正常输入的排列组合总数。
        """
        data = [[1, 2], [3, 4]]
        expected_total = 4  # 2! * 2! = 4
        self.assertEqual(calculate_total_permutations(data), expected_total)

    def test_single_row(self):
        """
        测试只有一行的二维数组。
        """
        data = [[1, 2, 3]]
        expected_total = 6  # 3! = 6
        self.assertEqual(calculate_total_permutations(data), expected_total)

    def test_multiple_rows(self):
        """
        测试多行，每行元素数量不同的情况。
        """
        data = [[1, 2], [3, 4], [5, 6]]
        # 每行有2个元素，所以排列组合总数是 2! * 2! * 2! = 8
        expected_total = 8
        self.assertEqual(calculate_total_permutations(data), expected_total)

    def test_empty_input(self):
        """
        测试空的二维数组。
        """
        with self.assertRaises(ValueError):
            calculate_total_permutations([])

    def test_empty_row(self):
        """
        测试包含空行的二维数组。
        """
        with self.assertRaises(ValueError):
            calculate_total_permutations([[], [1, 2]])

    def test_non_integer_elements(self):
        """
        测试包含非整数元素的二维数组。
        """
        data = [["a", "b"], ["c", "d"]]
        expected_total = 4  # 2! * 2! = 4
        self.assertEqual(calculate_total_permutations(data), expected_total)


class TestEvaluateListSimilarity(unittest.TestCase):
    def test_mse(self):
        """
        测试均方误差 (MSE) 评分方法。
        """
        lst = [1, 3, 6, 10]
        n = 2
        expected_score = 1.6666666666666667
        score = evaluate_list_similarity(lst, n, method="MSE")
        self.assertAlmostEqual(score, expected_score, places=7)

    def test_mae(self):
        """
        测试平均绝对误差 (MAE) 评分方法。
        """
        lst = [1, 3, 6, 10]
        n = 2
        expected_score = 1
        score = evaluate_list_similarity(lst, n, method="MAE")
        self.assertAlmostEqual(score, expected_score, places=7)

    def test_total_score(self):
        """
        测试总分评分方法。
        """
        lst = [1, 3, 6, 10]
        n = 2
        expected_score = 3
        score = evaluate_list_similarity(lst, n, method="total_score")
        self.assertAlmostEqual(score, expected_score, places=7)

    def test_empty_list(self):
        """
        测试传入空列表的情况。
        """
        lst = []
        n = 2
        expected_score = 0
        score = evaluate_list_similarity(lst, n, method="MSE")
        self.assertEqual(score, expected_score)

    def test_invalid_list_type(self):
        """
        测试传入无效的列表类型。
        """
        with self.assertRaises(TypeError):
            evaluate_list_similarity(None, 2, method="MSE")
        with self.assertRaises(TypeError):
            evaluate_list_similarity("string", 2, method="MSE")

    def test_invalid_element_type(self):
        """
        测试列表中含有无效元素类型。
        """
        with self.assertRaises(TypeError):
            evaluate_list_similarity([1, 2, "a"], 2, method="MSE")

    def test_invalid_n_type(self):
        """
        测试传入无效的 n 类型。
        """
        with self.assertRaises(TypeError):
            evaluate_list_similarity([1, 2, 3], "string", method="MSE")

    def test_invalid_method(self):
        """
        测试传入无效评分方法的情况。
        """
        with self.assertRaises(ValueError):
            evaluate_list_similarity([1, 2, 3], 2, method="INVALID_METHOD")


class TestCheckUnique(unittest.TestCase):
    def test_unique_elements(self):
        """
        测试所有元素唯一的情况。
        """
        data = [[1, 2, 3], [4, 5, 6]]
        self.assertTrue(check_unique(data))

    def test_non_unique_elements(self):
        """
        测试存在重复元素的情况。
        """
        data = [[1, 2, 3], [3, 4, 5]]
        self.assertFalse(check_unique(data))

    def test_empty_list(self):
        """
        测试空二维列表的情况。
        """
        data = []
        self.assertTrue(check_unique(data))  # 空列表视为唯一

    def test_empty_sublist(self):
        """
        测试包含空子列表的情况。
        """
        data = [[], [1, 2, 3]]
        self.assertTrue(check_unique(data))  # 仅有非空子列表，所以视为唯一

    def test_empty_and_non_empty_sublists(self):
        """
        测试包含空子列表和非空子列表的情况。
        """
        data = [[], [1, 1, 2]]
        self.assertFalse(check_unique(data))  # 含有重复元素

    def test_invalid_input_type(self):
        """
        测试传入非法输入类型的情况。
        """
        with self.assertRaises(TypeError):
            check_unique(123)  # 不是列表
        with self.assertRaises(TypeError):
            check_unique([[1, 2], 3])  # 第二个元素不是列表
        with self.assertRaises(TypeError):
            check_unique([[1, 2], [3, [4]]])  # 内部元素不是整数或浮点数


if __name__ == "__main__":
    unittest.main()



import json
import subprocess
from datetime import datetime, timedelta


def get_monthly_downloads(package, start_date, end_date):
    """
    获取按月的下载量，区分镜像和官网

    :param package: 库名称 (str)
    :param start_date: 开始日期 (datetime.date)
    :param end_date: 结束日期 (datetime.date)
    :return: (monthly_downloads, total_with_mirrors, total_without_mirrors)
    """
    current_date = start_date
    monthly_downloads = {}
    total_with_mirrors = 0
    total_without_mirrors = 0

    while current_date < end_date:
        # 计算下个月的起始日期
        next_month = current_date.replace(day=28) + timedelta(days=4)
        next_month = next_month.replace(day=1)
        query_end_date = min(next_month, end_date)

        cmd = [
            "pypistats",
            "overall",
            package,
            "--start-date",
            current_date.isoformat(),
            "--end-date",
            query_end_date.isoformat(),
            "--json",
        ]
        try:
            result = subprocess.run(
                cmd, capture_output=True, text=True, encoding="utf-8-sig", check=True
            )
        except subprocess.CalledProcessError:
            monthly_downloads[current_date.strftime("%Y-%m")] = {
                "with_mirrors": 0,
                "without_mirrors": 0,
                "total": 0,
            }
            current_date = next_month
            continue
        except Exception:
            monthly_downloads[current_date.strftime("%Y-%m")] = {
                "with_mirrors": 0,
                "without_mirrors": 0,
                "total": 0,
            }
            current_date = next_month
            continue

        try:
            data = json.loads(result.stdout)
        except json.JSONDecodeError:
            monthly_downloads[current_date.strftime("%Y-%m")] = {
                "with_mirrors": 0,
                "without_mirrors": 0,
                "total": 0,
            }
            current_date = next_month
            continue

        downloads_with_mirrors = 0
        downloads_without_mirrors = 0

        if "data" in data:
            for entry in data["data"]:
                try:
                    if entry.get("category") == "with_mirrors":
                        downloads_with_mirrors = int(entry.get("downloads", 0))
                    elif entry.get("category") == "without_mirrors":
                        downloads_without_mirrors = int(entry.get("downloads", 0))
                except (ValueError, TypeError):
                    pass

        month_label = current_date.strftime("%Y-%m")
        monthly_downloads[month_label] = {
            "with_mirrors": downloads_with_mirrors,
            "without_mirrors": downloads_without_mirrors,
            "total": downloads_with_mirrors,  # 按原逻辑，总数取 with_mirrors 的值
        }
        total_with_mirrors += downloads_with_mirrors
        total_without_mirrors += downloads_without_mirrors

        current_date = next_month

    return monthly_downloads, total_with_mirrors, total_without_mirrors


def fetch_download_stats(
    package, start_date, end_date, output_path=None, show_monthly=True
):
    """
    主函数接口，供外部直接调用。传入库名称、起止日期，
    返回各月下载量及总数，同时支持：
      - 将结果输出到指定路径下的TXT文件（当 output_path 不为 None 时）
      - 根据 show_monthly 参数决定是否输出每个月详细结果

    :param package: 库名称 (str)
    :param start_date: 开始日期 (datetime.date)
    :param end_date: 结束日期 (datetime.date)
    :param output_path: 输出文件路径 (str) 可选，若为 None 则直接打印输出到控制台
    :param show_monthly: 是否输出每个月的详细结果 (bool)
    :return: (monthly_downloads, total_with_mirrors, total_without_mirrors)
    """
    (
        monthly_downloads,
        total_with_mirrors,
        total_without_mirrors,
    ) = get_monthly_downloads(package, start_date, end_date)

    output_lines = []
    if show_monthly:
        output_lines.append("Monthly Downloads Breakdown:")
        for month, data in monthly_downloads.items():
            output_lines.append(
                f"{month}: with_mirrors={data['with_mirrors']}, without_mirrors={data['without_mirrors']}, total={data['total']}"
            )
    output_lines.append("\nTotal Downloads:")
    output_lines.append(f"With mirrors: {total_with_mirrors}")
    output_lines.append(f"Without mirrors: {total_without_mirrors}")
    output_lines.append(f"Overall total: {total_with_mirrors}")
    output_str = "\n".join(output_lines)

    if output_path:
        try:
            with open(output_path, "w", encoding="utf-8") as f:
                f.write(output_str)
        except Exception as e:
            print(f"写入文件失败: {e}")
            print(output_str)
    else:
        print(output_str)

    return monthly_downloads, total_with_mirrors, total_without_mirrors


if __name__ == "__main__":
    package = "myz_tools"
    start_date = datetime(2024, 8, 1).date()
    end_date = datetime.now().date()
    # 若需输出详细每月数据，则 show_monthly 设置为 True；若只想输出总计，则设置为 False
    # fetch_download_stats(package, start_date, end_date, output_path="output.txt", show_monthly=True)
    fetch_download_stats(package, start_date, end_date, show_monthly=False)



import itertools
import math
import os

import numpy as np
import pandas as pd


# 单元测试通过
def create_dir(dir_name, path="."):
    """
    在指定路径下创建名称为{dir_name}的文件夹
    Args:
        dir_name: 文件夹名称
        path: 要创建文件夹的路径，默认为当前路径

    Returns:
        无
    """
    full_path = os.path.join(path, dir_name)
    try:
        if not os.path.exists(full_path):
            os.makedirs(full_path)
            # print(f"文件夹 '{dir_name}' 已成功创建于 {path}")
        else:
            # print(f"文件夹 '{dir_name}' 已经存在于 {path}")
            pass
    except OSError as e:
        print(f"创建文件夹 '{dir_name}' 失败: {e}")


# 单元测试通过
def get_max_diff(two_dimensional_array):
    """
    参数：
        two_dimensional_array: 二维数组
    返回值：
        每一列里面最大值和最小值的差值，类型是一个一维数组
    功能：
        传入一个二维数组，函数返回每一列里面最大值和最小值的差值。
    """
    # 将输入转换为 NumPy 数组，并确保其为二维数组
    data = np.asarray(two_dimensional_array)

    if data.ndim != 2:
        raise ValueError("输入必须是一个二维数组或可以转换为二维数组的结构")

    # 使用 NumPy 的 ptp 函数直接计算每列的最大值和最小值差值
    diff = np.ptp(data, axis=0)

    return diff


# 单元测试通过
def remove_outliers_iqr(data):
    """
    参数：
        data: 二维数组
    返回值：
        去除异常值后的二维数组和有效的行索引，类型是一个元组
    功能：
        四分位距法去除传入的二维数组中的异常值，注意是对于每一列来说的自己的异常值
    """
    data = np.asarray(data)

    # 记录哪些行满足条件
    valid_rows = np.ones(data.shape[0], dtype=bool)

    for col in range(data.shape[1]):
        # 计算Q1和Q3
        Q1 = np.percentile(data[:, col], 25)
        Q3 = np.percentile(data[:, col], 75)
        IQR = Q3 - Q1

        # 计算上下边界
        lower_bound = Q1 - 1.5 * IQR
        upper_bound = Q3 + 1.5 * IQR

        # 更新valid_rows数组，保留不包含异常值的行
        valid_rows &= (data[:, col] >= lower_bound) & (data[:, col] <= upper_bound)

    # 筛选出有效的行
    cleaned_data = data[valid_rows]

    # 返回去除异常值后的数组和行索引
    valid_indices = np.where(valid_rows)[0]

    return cleaned_data, valid_indices


# 单元测试通过
def export_to_csv(array_data, file_name, output_directory="."):
    """
    参数：
        array_data: 二维数组，要保存的数据
        file_name: 字符串，CSV文件的名称（不包含扩展名）
        output_directory: 字符串，保存文件的目录路径，默认为当前目录
    返回：
        None
    功能：
        将给定的二维数组保存到指定目录中的CSV文件。如果文件已存在，则追加数据，并在每次写入时添加空行作为分隔符。
    """
    try:
        array_data = np.asarray(array_data)
        # 确保输入的是二维数组
        if array_data.ndim != 2:
            raise ValueError("输入必须是一个二维数组或可以转换为二维数组的结构")
        # 确保输出目录存在
        os.makedirs(output_directory, exist_ok=True)

        # 构建完整的文件路径
        full_file_path = os.path.join(output_directory, file_name + ".csv")

        # 判断文件是否存在并设置写入模式
        write_mode = "a" if os.path.exists(full_file_path) else "w"

        # 将二维数组转换为DataFrame并保存为CSV文件
        df = pd.DataFrame(array_data)
        df.to_csv(full_file_path, mode=write_mode, index=False, header=False)

        # 在文件末尾添加空行以分隔数据块
        with open(full_file_path, "a", encoding="utf-8") as file:
            file.write("\n")

    except Exception as error:
        raise IOError(f"保存数据到CSV文件失败: {str(error)}")


# TAG 0.2.0 更新


# 单元测试通过
def generate_2d_combinations_iter(data_3d, test_first_group=False):
    """
    功能：
        传入一个三维数组，返回每个二维数组的所有元素组合的迭代器。如果设置了 `test_first_group` 参数True,
        则仅返回第一个二维数组的组合。

    参数：
        data_3d (list): 一个三维列表，每个元素都是一个二维列表，表示多个二维数组。
        test_first_group (bool): 是否只生成第一个二维数组的组合。默认为 False。

    返回值：
        generator: 一个生成器对象，按需生成每个二维数组的所有元素组合。

    异常：
        ValueError: 如果输入的数据不是三维列表，则抛出异常。

    示例：
        >>> data = [[[1, 2], [3, 4]], [[5, 6], [7, 8]]]
        >>> result_iter = generate_2d_combinations_iter(data)
        >>> for combinations in result_iter:
                # 每次拿到是一个二维列表的组合
        ...     print(list(combinations)) # [(1, 3), (1, 4), (2, 3), (2, 4)]
        下一次
        [(5, 7), (5, 8), (6, 7), (6, 8)]


        >>> result_iter = generate_2d_combinations_iter(data, test_first_group=True)
        >>> print(list(next(result_iter)))
        [(1, 3), (1, 4), (2, 3), (2, 4)]
    """
    # 检查输入是否为三维列表
    if not all(isinstance(item, list) for sublist in data_3d for item in sublist):
        raise ValueError("输入的数据必须是一个三维列表")
    if not data_3d or not any(data_3d):
        raise ValueError("输入的三维列表不能为空")

    # 根据 test_first_group 参数决定处理方式
    data_to_process = [data_3d[0]] if test_first_group else data_3d

    for data_2d in data_to_process:
        # 使用 itertools.product 生成所有组合，返回迭代器
        yield itertools.product(*data_2d)


# 单元测试通过
def generate_row_permutations(data_2d):
    """
    功能：
        传入一个二维数组，返回该二维数组中每一行元素所有可能的排列组合。


    参数：
        data_2d (list): 一个二维列表，表示多个行数据。

    返回值：
        generator: 生成器对象，按需生成每一行的所有排列组合并组合成二维数组。

    异常：
        ValueError: 如果输入的二维数组为空或某行为空，则抛出异常。

    示例：
        >>> data = [[1, 2], [3, 4]]
        >>> permutations = generate_row_permutations(data)
        >>> for perm in permutations:
        ...     print(perm)
        [[1, 2], [3, 4]]
        [[1, 2], [4, 3]]
        [[2, 1], [3, 4]]
        [[2, 1], [4, 3]]
    """
    if not data_2d or not all(data_2d):
        raise ValueError("输入的二维数组不能为空，且每一行都应包含至少一个元素。")

    row_permutations = [itertools.permutations(row) for row in data_2d]
    # 一次 row:[1,2]  -> row_permutations: [(1, 2), (2, 1)]

    for combination in itertools.product(*row_permutations):
        yield [list(row) for row in combination]


# 单元测试通过
def calculate_total_permutations(data_2d):
    """
    功能：
        计算并返回二维数组中所有行的排列组合总数。

    参数：
        data_2d (list): 一个二维列表，表示多个行数据。

    返回值：
        int: 所有行排列组合的总数。

    异常：
        ValueError: 如果输入的二维数组为空或某行为空，则抛出异常。

    示例：
        >>> data = [[1, 2], [3, 4]]
        >>> total = calculate_total_permutations(data)
        >>> print(total)
        4
    """
    if not data_2d or not all(data_2d):
        raise ValueError("输入的二维数组不能为空，且每一行都应包含至少一个元素。")

    num_permutations = math.factorial(len(data_2d[0]))
    # 一维列表长度为n,则排列组合数为n!，得到的一维组合列表的长度是n!
    # n!*n!*n!...n!总共是行数个，所以下面乘以行数
    total = num_permutations ** len(data_2d)
    # print(f"总排列组合数: {total}")
    return total


# 不测试
def display_combinations(data_2d, N=5):
    """
    功能：
        打印二维数组中前 N 个排列组合，以检测组合生成的正确性。

    参数：
        data_2d (list): 一个二维列表，表示多个行数据。
        N (int): 要打印的组合数量，默认为 5。

    返回值：
        None

    异常：
        ValueError: 如果输入的二维数组为空或某行为空，则抛出异常。

    示例：
        >>> data = [[1, 2], [3, 4]]
        >>> display_combinations(data, N=2)
        [1, 2]
        [3, 4]

        [1, 2]
        [4, 3]
    """
    if not data_2d or not all(data_2d):
        raise ValueError("输入的二维数组不能为空，且每一行都应包含至少一个元素。")

    combinations = generate_row_permutations(data_2d)

    for i, combination in enumerate(combinations):
        if i < N:
            for row in combination:
                print(row)
            print()
        else:
            break


# 单元测试通过
def evaluate_list_similarity(lst, n, method="MSE"):
    """
    功能：
        评估列表中元素间的差异与给定值 `n` 的接近程度。支持不同的评分方法，例如均方误差 (MSE)、绝对误差等。
        三种方式都是得分越小表示列表元素与目标值的接近程度越高
    参数：
        lst (list of int/float): 需要评估的数字列表。
        n (int/float): 用于比较的参考值，计算差异的目标值。
        method (str): 选择的评分方法。支持 'MSE'（均方误差）、'MAE'（平均绝对误差）、'total_score'（总分）等。

    返回值：
        float: 计算得出的评分值。

    异常：
        ValueError: 如果输入的 `lst` 不是列表或 `n` 不是整数或浮点数，则抛出异常。
        TypeError: 如果 `lst` 中的元素不是整数或浮点数，则抛出异常。
        ValueError: 如果选择的评分方法不被支持，则抛出异常。

    示例：
        >>> lst = [1, 3, 6, 10]
        >>> n = 2
        >>> score = evaluate_list_similarity(lst, n, method='MSE')
        >>> print(score)

        >>> score = evaluate_list_similarity(lst, n, method='MAE')
        >>> print(score)

    """
    # 验证输入的有效性
    if not isinstance(lst, list) or not all(isinstance(x, (int, float)) for x in lst):
        raise TypeError("输入的列表 `lst` 必须是包含整数或浮点数的列表。")
    if not isinstance(n, (int, float)):
        raise TypeError("参数 `n` 必须是整数或浮点数。")
    if method not in ["MSE", "MAE", "total_score"]:
        raise ValueError(f"不支持的评分方法 '{method}'。请选择 'MSE'、'MAE' 或 'total_score'。")

    # 对列表进行排序
    sorted_lst = sorted(lst)

    # 计算相邻元素之间的差值
    differences = [
        abs(sorted_lst[i + 1] - sorted_lst[i]) for i in range(len(sorted_lst) - 1)
    ]

    if not differences:
        return 0

    if method == "MSE":
        # 计算均方误差 (MSE)
        squared_errors = [(diff - n) ** 2 for diff in differences]
        score = np.mean(squared_errors)

    elif method == "MAE":
        # 计算平均绝对误差 (MAE)
        absolute_errors = [abs(diff - n) for diff in differences]
        score = np.mean(absolute_errors)

    elif method == "total_score":
        # 计算总分
        total_score = sum(abs(diff - n) for diff in differences)
        score = total_score

    return score


# 单元测试通过
def check_unique(data):
    """
    功能：
        检查传入的二维数组中的每个元素是否都是唯一的。如果所有元素唯一，返回 True；否则，返回 False。

    参数：
        data (list of list): 一个二维列表，其中包含需要检查的元素。

    返回值：
        bool: 如果所有元素都唯一，返回 True；否则，返回 False。

    异常：
        TypeError: 如果输入的 `data` 不是一个二维列表，或包含非列表的元素，则抛出异常。

    示例：
        >>> data = [[1, 2, 3], [4, 5, 6]]
        >>> check_unique(data)
        True

        >>> data = [[1, 2, 3], [3, 4, 5]]
        >>> check_unique(data)
        False
    """
    # 验证输入是否为二维列表
    if not isinstance(data, list) or not all(
        isinstance(sublist, list) for sublist in data
    ):
        raise TypeError("输入必须是一个二维列表。")

    # 将二维数组展平为一维数组
    flattened_data = [item for sublist in data for item in sublist]

    # 检查是否所有元素都是唯一的
    return len(flattened_data) == len(set(flattened_data))


if __name__ == "__main__":
    pass



# 导入ast模块，用于解析Python源代码
import ast
# 导入inspect模块，用于获取源代码
import inspect
import json
import os


# TAG下，第三版，在第二版的基础上增加对文件里面所有的python文件处理，
# #################### 并且支持选择放到同一个md文件还是按照python文件的名称放到指定目录下的不同md文件
def dir2md(source_dir, output_dir="./MD/", single_file=True):
    """
    Args:
        source_dir: 待处理目录
        output_dir: 输出目录
        single_file: 是否将所有文件放到一个md文件中
    """
    if single_file:
        # 要放到一个md文件里面
        # 拿到source_dir的名称

        source_dir_name = os.path.basename(source_dir)
        output_path = output_dir + source_dir_name + ".md"
        mode = "w"

        for root, dirs, files in os.walk(source_dir):
            for file in files:
                if file.endswith(".py"):
                    # 获取文件的绝对路径
                    file_path = os.path.join(root, file)
                    # 拿到文件信息
                    function_docs, class_info = extract_info(file_path)
                    all_save_markdown(
                        file_path, output_path, function_docs, class_info, mode
                    )
                    mode = "a"

        pass
    else:
        # 要放到指定目录下的不同md文件，文件名称为自己项目中的python文件名称
        # 遍历source_dir目录下的所有python文件
        for root, dirs, files in os.walk(source_dir):
            for file in files:
                if file.endswith(".py"):
                    # 获取文件的绝对路径
                    file_path = os.path.join(root, file)
                    # 拿到该文件的名称
                    file_name = os.path.basename(file_path)
                    # 后缀修改为md
                    md_file_name = os.path.splitext(file_name)[0] + ".md"
                    output_path = output_dir + md_file_name
                    all2md(file_path, output_path)

    pass


# TAG 下：第二版
# 解析、保存、调用
def extract_info(file_path):
    """
    功能：
        解析python文件，提取函数和类信息
    Args:
        file_path: 待处理文件路径
    Returns:
        function_docs: 函数信息
        class_info: 类信息
    """
    with open(file_path, "r", encoding="utf-8") as file:
        source = file.read()

    # 检测是否栈溢出
    try:
        # 将源代码解析为抽象语法树
        tree = ast.parse(source)
    except RecursionError as e:
        print(f"当前处理的{file_path}文件太大，栈溢出，请切割文件后重新调用: {e}")
        return None, None

    # 创建一个字典，用于存储函数的文档字符串
    function_docs = {}
    # 创建一个字典，用于存储类的信息
    class_info = {}
    # 一个字典，用于判断函数的时候跳过类函数，采用函数名 + 注释双重判定
    # HACK 不是很优雅，还可以继续优化这个逻辑
    skip_class_fun = {}

    for node in ast.walk(tree):
        # 类，以及类方法
        if isinstance(node, ast.ClassDef):
            class_name = node.name
            class_docstring = ast.get_docstring(node)
            class_info[class_name] = {"docstring": class_docstring, "members": []}
            for member in node.body:
                if isinstance(member, ast.FunctionDef):
                    # 处理类中的方法
                    func_name = member.name
                    func_docstring = ast.get_docstring(member)
                    class_info[class_name]["members"].append(
                        {
                            "type": "method",
                            "name": func_name,
                            "docstring": func_docstring,
                        }
                    )
                    # 判定条件是：类方法名 + 注释
                    skip_class_fun[func_name] = func_docstring
                elif isinstance(member, ast.Assign):
                    # 处理类的属性
                    for target in member.targets:
                        if isinstance(target, ast.Name):
                            attr_name = target.id
                            class_info[class_name]["members"].append(
                                {"type": "attribute", "name": attr_name}
                            )
        # 普通函数
        elif isinstance(node, ast.FunctionDef):
            func_name = node.name
            func_docstring = ast.get_docstring(node)
            # HACK 不是很优雅，还可以继续优化这个逻辑
            # 根据方法名+注释信息判断是否为类方法，为类方法的话就直接跳过访问
            check_key_value = (func_name, func_docstring)
            if check_key_value in skip_class_fun.items():
                continue
            function_docs[func_name] = func_docstring
    return function_docs, class_info


def all_save_markdown(file_path, output_path, function_docs, class_info, mode="w"):
    """
    功能：
        将提取的函数和类信息保存为markdown文件
    Args:
        file_path: 待处理文件路径
        output_path: 保存路径
        function_docs: 函数信息
        class_info: 类信息
    Returns:
        无

    """
    with open(output_path, mode=mode, encoding="utf-8") as file:
        file.write("# 文件名称: " + os.path.basename(file_path) + "\n")
        file.write("***\n")
        if mode == "w":
            file.write("[TOC]\n")
        file.write("***\n")
        file.write("\n## 函数部分：\n")
        for func_name, docstring in function_docs.items():
            file.write(f"#### 函数{func_name}\n")
            if docstring:
                file.write(f"```\n{docstring}\n```\n")
            else:
                file.write("\n")
        file.write("***\n")

        file.write("\n## 类部分：\n")
        for class_name, class_info in class_info.items():
            file.write(f"### 类{class_name}\n")
            if class_info["docstring"]:
                file.write(f"```\n{class_info['docstring']}\n```\n")
            else:
                file.write("\n")
            for member in class_info["members"]:
                if member["type"] == "method":
                    file.write(f"#### 类方法{member['name']}\n")
                    if member["docstring"]:
                        file.write(f"```\n{member['docstring']}\n```\n")
                    else:
                        file.write("\n")
                elif member["type"] == "attribute":
                    file.write(f"#### attribute_name:{member['name']}\n")
        file.write("***\n")

        pass
    pass


def all2md(file_path, output_path):
    """
    功能：调用，将一个python文件中的函数和类信息提取出来，并保存为markdown文件
    Args:
        file_path: 字符串，python文件的路径
        output_path: 字符串，markdown文件的保存路径
    Returns:
        无
    """
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"File '{file_path}' not found.")
    function_docs, class_info = extract_info(file_path)
    if function_docs or class_info:
        all_save_markdown(file_path, output_path, function_docs, class_info)
    else:
        print(f"文件{file_path}存在没有函数或类定义，或者是有但是文件本身太大，无法解析")
        # raise Exception("文件没有函数或类定义，或者是有但是文件本身太大，无法解析")
        pass
    pass


# TAG 下：第一版
def extract_function_docs_from_file(file_path):
    """
    从 Python 文件中提取函数的 docstring（注释部分）。

    参数:
        file_path: 字符串，Python 文件的路径。

    返回:
        字典，键为函数名，值为函数的 docstring。
    """

    with open(file_path, "r", encoding="utf-8") as file:
        source = file.read()

    # 解析源代码
    tree = ast.parse(source)

    function_docs = {}

    # 遍历 AST 节点，提取函数定义和 docstring
    for node in ast.walk(tree):
        if isinstance(node, ast.FunctionDef):
            func_name = node.name
            docstring = ast.get_docstring(node)
            function_docs[func_name] = docstring

    return function_docs


def save_docs_to_markdown(docs, output_path):
    """
    将函数的 docstring 保存到 Markdown 文件中。

    参数:
        docs: 字典，包含函数名和 docstring 的映射。
        output_path: 字符串，Markdown 文件的保存路径。
    """
    # if os.path.exists(output_path):
    #     mode = "a"
    # else:
    #     mode = "w"
    with open(output_path, mode="w", encoding="utf-8") as file:
        for func_name, docstring in docs.items():
            file.write(f"### {func_name}\n\n")
            if docstring:
                file.write(f"```\n{docstring}\n```\n")
            else:
                file.write("无 docstring\n")
            file.write("\n")


def pyFun2md(source_file, output_md):
    """
    将 Python 文件中提取 docstring 并保存为 Markdown 文件。

    参数:
        source_file: 字符串，源 Python 文件的路径。
        output_md: 字符串，输出 Markdown 文件的路径。
    """
    if not os.path.exists(source_file):
        raise FileNotFoundError(f"文件 {source_file} 不存在")
    docs = extract_function_docs_from_file(source_file)
    save_docs_to_markdown(docs, output_md)


# TAG 简单测试位置
if __name__ == "__main__":
    # 第一版，只能对python里普通函数处理
    # source_file_path = "./myz_tools/common_maths.py"  # Python 文件路径
    # output_md_path = "./MD/common_maths_doc.md"  # 输出 Markdown 文件路径
    # pyFun2md(source_file_path, output_md_path)

    # # 第二版，可以对python里普通函数和类方法处理
    # source_file_path = "./myz_tools/common_maths.py"  # Python 文件路径
    # output_md_path = "./MD/common_maths_doc.md"  # 输出 Markdown 文件路径
    # all2md(source_file_path, output_md_path)

    # source_file_path = "./myz_tools/all_test.py"  # Python 文件路径
    # output_md_path = "./MD/all_test_doc.md"  # 输出 Markdown 文件路径
    # all2md(source_file_path, output_md_path)

    # 第三版，可以处理一个目录下的所有python文件，并且可以选择是否将所有文件放到一个md文件中
    # # 分别放到单个文件测试
    # source_dir = "./myz_tools"  # Python 项目目录
    # output_dir = "./MD/"  # 输出目录
    # dir2md(source_dir, output_dir, single_file=False)

    # # 放到一个文件中测试
    # source_dir = "./myz_tools"  # Python 项目目录
    # output_dir = "./MD/"  # 输出目录
    # dir2md(source_dir, output_dir, single_file=True)

    # 放到一个文件中测试，使用默认参数
    source_dir = "./myz_tools"  # Python 项目目录
    dir2md(source_dir)

    # 第三版简单测试通过
    pass



# # 导入setuptools模块，用于打包和分发Python项目
# # 导入re模块，用于正则表达式操作
# import re

# # 导入requests模块，用于发送HTTP请求
# import requests
# import setuptools
# # 导入BeautifulSoup模块，用于解析HTML文档
# from bs4 import BeautifulSoup

# package_name = "myz_tools"


# def curr_version():
#     """
#     参数：无
#     返回值：当前官网上最新的版本号，格式为x.y.z
#     原理是从PyPi网站上面获取最新的版本号
#     """

#     # 从官网获取版本号
#     # 定义URL，使用f-string格式化字符串，将package_name插入URL中
#     url = f"https://pypi.org/project/{package_name}/"
#     # url = f"https://test.pypi.org/project/{package_name}/"

#     # 请求头

#     # headers = {
#     #     "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36 Edg/127.0.0.0"
#     # }
#     # # 发送GET请求，获取响应
#     # response = requests.get(url, headers=headers)

#     # # 检查是否响应成功
#     # if response.status_code == 200:
#     #     try:
#     #         print(f"成功获取官网的{package_name}的响应，响应code：{response.status_code}")
#     #         # 使用BeautifulSoup解析响应内容
#     #         soup = BeautifulSoup(response.content, "html.parser")

#     #         print(f"获取到的soup内容是：{soup}")

#     #         # 使用CSS选择器，选择.release__version类下的第一个元素，获取其文本内容，并去除首尾空格
#     #         latest_version = soup.select_one(".release__version").text.strip()
#     #         # print(f"当前PyPi上面的最新的版本号是：{latest_version}")
#     #         # 返回最新版本号

#     #         return str(latest_version)
#     #     except Exception as e:
#     #         print(f"test.pypi获取{package_name}的响应失败，错误信息：{e}")
#     #         print(
#     #             f"test.pypi获取{package_name}的响应失败，状态码：{response.status_code}，尝试从文件中获取版本号"
#     #         )
#     #         # 通过文件的存储版本号
#     #         with open("version.txt", "r", encoding="utf-8") as f:
#     #             # 读取所有行
#     #             lines = f.readlines()
#     #         # 获取最后一行
#     #         latest_version = lines[-1]
#     #         # 打印最新版本号
#     #         print(latest_version)
#     #         return str(latest_version)
#     #     pass

#     # 通过文件的存储版本号
#     with open("version.txt", "r", encoding="utf-8") as f:
#         # 读取所有行
#         lines = f.readlines()
#     # 获取最后一行
#     latest_version = lines[-1]
#     # 打印最新版本号
#     print(latest_version)

#     # latest_version = '0.2.2'

#     return str(latest_version)

#     pass


# def get_version():
#     """
#     参数：无
#     返回值：由官网上的最新的版本好计算出当前最新的版本号，格式为x.y.z
#     """

#     # 从版本号字符串中提取三个数字并将它们转换为整数类型
#     # 使用正则表达式匹配当前版本号
#     match = re.search(r"(\d+)\.(\d+)\.(\d+)", curr_version())
#     # 将匹配到的第一个数字转换为整数，作为主版本号
#     major = int(match.group(1))
#     # 将匹配到的第二个数字转换为整数，作为次版本号
#     minor = int(match.group(2))
#     # 将匹配到的第三个数字转换为整数，作为补丁号
#     patch = int(match.group(3))

#     # 对三个数字进行加一操作
#     patch += 1
#     # 如果patch大于9，则将patch置为0，并将minor加一
#     if patch > 9:
#         patch = 0
#         minor += 1
#         # 如果minor大于9，则将minor置为0，并将major加一
#         if minor > 9:
#             minor = 0
#             major += 1
#     # 将major、minor、patch拼接成新的版本号字符串
#     new_version_str = f"{major}.{minor}.{patch}"
#     # 返回新的版本号字符串

#     # # # 手动指定版本号
#     # new_version_str = '0.2.1'

#     return new_version_str


# def upload():
#     # 打开README.md文件，以只读模式读取文件内容
#     with open("README.md", "r", encoding="utf-8", errors="ignore") as fh:
#         long_description = fh.read()
#     # 打开requirements.txt文件，读取文件内容，并按行分割
#     with open("requirements.txt", encoding="utf-8", errors="ignore") as f:
#         required = f.read().splitlines()

#     setuptools.setup(
#         name=package_name,
#         version=get_version(),
#         author="宇千思",  # 作者名称
#         author_email="",  # 作者邮箱
#         description="Python helper tools",  # 库描述
#         long_description=long_description,  # 设置长描述
#         long_description_content_type="text/markdown",  # 设置长描述的内容类型为Markdown
#         url="https://pypi.org/project/myz_tools/",  # 库的官方地址
#         # 查找项目中的所有包
#         packages=setuptools.find_packages(include=["myz_tools", "myz_tools.*"]),
#         data_files=["requirements.txt"],  # myz_tools库依赖的其他库
#         classifiers=[
#             "License :: OSI Approved :: Apache Software License",  # 授权协议
#         ],
#         # 指定Python版本要求，这里要求Python版本大于等于
#         python_requires=">=3.6",
#         # 指定安装依赖，这里使用required变量，该变量应该是一个包含所有依赖的列表
#         install_requires=required,
#     )


# # 定义一个函数，用于写入当前版本号
# def write_now_version():
#     # 打开名为VERSION的文件，以写入模式
#     with open("version.txt", "a", encoding="utf-8", errors="ignore") as version_f:
#         # 将当前版本号写入文件
#         version_f.write(get_version())
#         # 写完后换行
#         version_f.write("\n")
#         pass


# def main():
#     try:
#         # 尝试执行上传操作
#         upload()
#         write_now_version()
#         # 打印上传成功信息，并显示当前版本号
#         print("已经成功准备好包，可以进行打包操作, 更新后的最新版本号将是:", get_version())
#     except Exception as e:
#         # 如果上传过程中出现异常，则抛出异常，并显示异常信息
#         raise Exception("上传出现异常", e)


# if __name__ == "__main__":
#     main()
#     pass


# 说明：
# 1. 程序首先尝试从官网获取最新版本号，如果失败则从 version.txt 文件中获取最后一行符合 x.y.z 格式的版本号，
#    并输出“文件当前版本: 当前版本号”。
# 2. 版本号的更新规则为：基于当前版本号的补丁号加 1（当补丁号超过 9 时，进位至次版本号；依此类推）。
# 3. 允许通过命令行手动指定版本号，使用指令：python setup.py --manual-version=1.2.3
#    手动指定后，程序将直接使用该版本号，不再自动计算。

import re
import sys

import requests
import setuptools
from bs4 import BeautifulSoup

package_name = "myz_tools"
_current_version_cache = None
manual_version = None


def curr_version():
    """
    获取当前版本号：
    首先尝试从官网获取，如果出错则从 version.txt 文件中读取最后一行符合 x.y.z 格式的版本号，
    并输出文件当前版本。
    """
    global _current_version_cache
    if _current_version_cache is not None:
        return _current_version_cache

    url = f"https://pypi.org/project/{package_name}/"
    headers = {
        "User-Agent": (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/127.0.0.0 Safari/537.36 Edg/127.0.0.0"
        )
    }
    try:
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            soup = BeautifulSoup(response.content, "html.parser")
            latest_version = soup.select_one(".release__version").text.strip()
            _current_version_cache = latest_version
            return _current_version_cache
    except Exception:
        pass

    # 官网获取失败，从 version.txt 中读取
    with open("version.txt", "r", encoding="utf-8") as f:
        valid_lines = [
            line.strip().lstrip("\ufeff")
            for line in f
            if re.match(r"^\d+\.\d+\.\d+$", line.strip().lstrip("\ufeff"))
        ]
    if not valid_lines:
        raise ValueError("version.txt文件中没有有效版本号")
    _current_version_cache = valid_lines[-1]
    print(f"文件当前版本: {_current_version_cache}")
    return _current_version_cache


def get_version():
    """
    根据当前版本号计算更新后的版本号（x.y.z格式）。
    如果通过命令行手动指定了版本号，则直接返回该版本号。
    """
    global manual_version
    if manual_version:
        return manual_version

    current_ver = curr_version().strip()
    match = re.search(r"(\d+)\.(\d+)\.(\d+)", current_ver)
    if not match:
        raise ValueError("版本号格式不正确")
    major, minor, patch = int(match.group(1)), int(match.group(2)), int(match.group(3))
    patch += 1
    if patch > 9:
        patch = 0
        minor += 1
        if minor > 9:
            minor = 0
            major += 1
    return f"{major}.{minor}.{patch}"


def upload():
    with open("README.md", "r", encoding="utf-8", errors="ignore") as fh:
        long_description = fh.read()
    with open("requirements.txt", encoding="utf-8", errors="ignore") as f:
        required = f.read().splitlines()

    setuptools.setup(
        name=package_name,
        version=get_version(),
        author="宇千思",
        author_email="",
        description="Python helper tools",
        long_description=long_description,
        long_description_content_type="text/markdown",
        url="https://pypi.org/project/myz_tools/",
        packages=setuptools.find_packages(include=["myz_tools", "myz_tools.*"]),
        data_files=["requirements.txt"],
        classifiers=[
            "License :: OSI Approved :: Apache Software License",
        ],
        python_requires=">=3.6",
        install_requires=required,
    )


def write_now_version():
    with open("version.txt", "a", encoding="utf-8", errors="ignore") as version_f:
        version_f.write(get_version() + "\n")


def main():
    global manual_version
    new_args = []
    for arg in sys.argv:
        if arg.startswith("--manual-version="):
            manual_version = arg.split("=", 1)[1]
            print(f"手动指定版本号: {manual_version}")
        else:
            new_args.append(arg)
    sys.argv = new_args

    try:
        upload()
        write_now_version()
        print(f"更新后的版本号: {get_version()}")
    except Exception as e:
        raise Exception("上传出现异常", e)


if __name__ == "__main__":
    main()
